-- Main.lua

function setup()
    -- Set up the screen
    viewer.mode = FULLSCREEN
    
    -- Initialize variables
    people = {}
    numPeople = 35
    emojiDuration = 1 -- seconds
    emojiSize = 50
    
    -- Create people
    for i = 1, numPeople do
        table.insert(people, {
            x = math.random(WIDTH),
            y = math.random(HEIGHT),
            size = 24,
            speed = math.random(50, 100),
            angle = math.random() * 2 * math.pi,
            emoji = nil,
            emojiTime = 0,
            wanderAngle = 0, -- For wandering behavior
            wanderAmount = 0.5 -- How quickly they change direction
        })
    end
end

function draw()
    -- Clear the screen
    background(0)
    
    -- Update and draw people
    for i, person in ipairs(people) do
        -- Wander behavior: Randomly change direction
        person.angle = person.angle + (math.random() - 0.5) * person.wanderAmount
        person.x = person.x + math.cos(person.angle) * person.speed * DeltaTime
        person.y = person.y + math.sin(person.angle) * person.speed * DeltaTime
        
        -- Bounce off walls
        if person.x < 0 or person.x > WIDTH then
            person.angle = math.pi - person.angle
        end
        if person.y < 0 or person.y > HEIGHT then
            person.angle = -person.angle
        end
        
        -- Draw the person
        fill(255)
        rect(person.x - person.size/2, person.y - person.size/2, person.size, person.size)
        
        -- Draw emoji if active
        if person.emoji and ElapsedTime - person.emojiTime < emojiDuration then
            textSize(emojiSize)
            text(person.emoji, person.x, person.y + person.size)
        else
            person.emoji = nil
        end
    end
    
    -- Check for collisions
    for i = 1, #people do
        for j = i + 1, #people do
            if checkCollision(people[i], people[j]) then
                -- Show emojis
                showEmoji(people[i])
                showEmoji(people[j])
                
                -- Bounce off each other
                bounce(people[i], people[j])
            end
        end
    end
end

function checkCollision(a, b)
    local dx = a.x - b.x
    local dy = a.y - b.y
    local distance = math.sqrt(dx * dx + dy * dy)
    return distance < a.size
end

function showEmoji(person)
    if not person.emoji then
        person.emoji = getRandomEmoji()
        person.emojiTime = ElapsedTime
    end
end

function getRandomEmoji()
    local emojis = {"😀", "😎", "🤖", "👻", "💩", "🎉", "❤️", "🔥", "🌈", "🍕"}
    return emojis[math.random(#emojis)]
end

function bounce(a, b)
    -- Calculate the vector between the two people
    local dx = b.x - a.x
    local dy = b.y - a.y
    local distance = math.sqrt(dx * dx + dy * dy)
    
    -- Normalize the vector
    local nx = dx / distance
    local ny = dy / distance
    
    -- Calculate the relative velocity
    local relativeSpeed = a.speed - b.speed
    
    -- Reflect the angles based on the collision
    a.angle = math.atan(math.sin(a.angle) - ny, math.cos(a.angle) - nx)
    b.angle = math.atan(math.sin(b.angle) + ny, math.cos(b.angle) + nx)
    
    -- Move them apart slightly to avoid sticking
    local overlap = a.size - distance
    a.x = a.x - nx * overlap / 2
    a.y = a.y - ny * overlap / 2
    b.x = b.x + nx * overlap / 2
    b.y = b.y + ny * overlap / 2
end