-- Main.lua

function setup()
    -- Set up the screen
    viewer.mode = FULLSCREEN
    
    -- Initialize variables
    people = {}
    numPeople = 55
    emojiDuration = 1.1 -- seconds
    emojiSize = 50
    emojiFloatSpeed = 5 -- pixels per second
    
    -- Create people
    for i = 1, numPeople do
        table.insert(people, {
            x = math.random(WIDTH),
            y = math.random(HEIGHT),
            size = 26,
            speed = math.random(50, 80),
            angle = math.random() * 2 * math.pi,
            emoji = nil,
            emojiTime = 0,
            emojiYOffset = 0, -- For floating animation
            color = color(math.random(255), math.random(255), math.random(255)), -- Random color
            wanderAngle = 0, -- For wandering behavior
            wanderAmount = 0.1, -- How quickly they change direction
            speedChangeTimer = 0, -- Timer for speed changes
            speedChangeInterval = math.random(2, 5) -- Interval between speed changes
        })
    end
end

function draw()
    -- Clear the screen
    background(0)
    
    -- Update and draw people
    for i, person in ipairs(people) do
        -- Wander behavior: Randomly change direction
        person.angle = person.angle + (math.random() - 0.5) * person.wanderAmount
        person.x = person.x + math.cos(person.angle) * person.speed * DeltaTime
        person.y = person.y + math.sin(person.angle) * person.speed * DeltaTime
        
        -- Bounce off walls
        if person.x < person.size / 2 or person.x > WIDTH - person.size / 2 then
            person.angle = math.pi - person.angle
        end
        if person.y < person.size / 2 or person.y > HEIGHT - person.size / 2 then
            person.angle = -person.angle
        end
        
        -- Draw the person with their color
        fill(person.color)
        rect(person.x - person.size/2, person.y - person.size/2, person.size, person.size)
        
        -- Draw emoji if active
        if person.emoji and ElapsedTime - person.emojiTime < emojiDuration then
            -- Calculate alpha (fade out over time)
            local alpha = 255 * (1 - (ElapsedTime - person.emojiTime) / emojiDuration)
            -- Float upward
            person.emojiYOffset = person.emojiYOffset + emojiFloatSpeed * DeltaTime
            -- Draw emoji with alpha
            fill(255, 255, 255, alpha)
            textSize(emojiSize)
            text(person.emoji, person.x, person.y + person.size + person.emojiYOffset)
        else
            person.emoji = nil
            person.emojiYOffset = 0 -- Reset offset
        end
        
        -- Occasionally change speed
        person.speedChangeTimer = person.speedChangeTimer + DeltaTime
        if person.speedChangeTimer > person.speedChangeInterval then
            person.speed = math.random(50, 100) -- New random speed
            person.speedChangeTimer = 0 -- Reset timer
            person.speedChangeInterval = math.random(2, 5) -- New random interval
        end

    end
    
    -- Check for collisions
    for i = 1, #people do
        for j = i + 1, #people do
            if checkCollision(people[i], people[j]) then
                -- Show emojis
                showEmoji(people[i], people[j])
                
                -- Bounce off each other
                bounce(people[i], people[j])
            end
        end
    end
end





function draw()
    -- Clear the screen
    background(0)
    
    -- First pass: Draw all people
    for i, person in ipairs(people) do
        -- Update position
        person.x = person.x + math.cos(person.angle) * person.speed * DeltaTime
        person.y = person.y + math.sin(person.angle) * person.speed * DeltaTime
        
        -- Bounce off walls
        if person.x < person.size / 2 or person.x > WIDTH - person.size / 2 then
            person.angle = math.pi - person.angle
        end
        if person.y < person.size / 2 or person.y > HEIGHT - person.size / 2 then
            person.angle = -person.angle
        end
        
        -- Draw the person with their color and rotation
        pushMatrix()
        translate(person.x, person.y)
        rotate(person.angle)
        fill(person.color)
        rect(-person.size / 2, -person.size / 2, person.size, person.size)
        popMatrix()
        
        -- Occasionally change speed
        person.speedChangeTimer = person.speedChangeTimer + DeltaTime
        if person.speedChangeTimer > person.speedChangeInterval then
            person.speed = math.random(50, 100) -- New random speed
            person.speedChangeTimer = 0 -- Reset timer
            person.speedChangeInterval = math.random(2, 5) -- New random interval
        end
    end
    
    -- Second pass: Draw all emojis
    for i, person in ipairs(people) do
        if person.emoji and ElapsedTime - person.emojiTime < emojiDuration then
            -- Calculate alpha (fade out over time)
            local alpha = 255 * (1 - (ElapsedTime - person.emojiTime) / emojiDuration)
            -- Float upward
            person.emojiYOffset = person.emojiYOffset + emojiFloatSpeed * DeltaTime
            -- Draw emoji with alpha
            fill(255, 255, 255, alpha)
            textSize(emojiSize)
            text(person.emoji, person.x, person.y + person.size + person.emojiYOffset)
        else
            person.emoji = nil
            person.emojiYOffset = 0 -- Reset offset
        end
    end
    
    -- Check for collisions
    for i = 1, #people do
        for j = i + 1, #people do
            if checkCollision(people[i], people[j]) then
                -- Show emojis
                showEmoji(people[i], people[j])
                
                -- Bounce off each other
                bounce(people[i], people[j])
            end
        end
    end
end





function checkCollision(a, b)
    local dx = a.x - b.x
    local dy = a.y - b.y
    local distance = math.sqrt(dx * dx + dy * dy)
    return distance < a.size
end

function showEmoji(person1, person2)
    if not person1.emoji then
        if math.random() < 0.9 then -- 90% chance of same emoji
            local sharedEmoji = getRandomEmoji()
            person1.emoji = sharedEmoji
            person2.emoji = sharedEmoji
        else -- 10% chance of different emojis
            person1.emoji = getRandomEmoji()
            person2.emoji = getRandomEmoji()
        end
        person1.emojiTime = ElapsedTime
        person2.emojiTime = ElapsedTime
        person1.emojiYOffset = 0
        person2.emojiYOffset = 0
    end
end

function getRandomEmoji()
    local emojis = {"😀", "😎", "🤖", "👻", "💩", "🎉", "❤️", "🔥", "🌈", "🍕"}
    return emojis[math.random(#emojis)]
end

function bounce(a, b)
    -- Calculate the vector between the two people
    local dx = b.x - a.x
    local dy = b.y - a.y
    local distance = math.sqrt(dx * dx + dy * dy)
    
    -- Normalize the vector
    local nx = dx / distance
    local ny = dy / distance
    
    -- Reflect the angles based on the collision
    a.angle = math.atan(math.sin(a.angle) - ny, math.cos(a.angle) - nx)
    b.angle = math.atan(math.sin(b.angle) + ny, math.cos(b.angle) + nx)
    
    -- Move them apart slightly to avoid sticking
    local overlap = a.size - distance
    a.x = a.x - nx * overlap / 2
    a.y = a.y - ny * overlap / 2
    b.x = b.x + nx * overlap / 2
    b.y = b.y + ny * overlap / 2
end